@@ -28,6 +28,7 @@
 #include <libavformat/avformat.h>
 /* #include <ffmpeg/avi.h> */
 #include <gst/gst.h>
+#include <gst/base/gstflowcombiner.h>
 
 #include "gstav.h"
 #include "gstavcodecmap.h"
@@ -49,7 +50,6 @@ struct _GstFFStream
   GstClockTime last_ts;
   gboolean discont;
   gboolean eos;
-  GstFlowReturn last_flow;
 
   GstTagList *tags;             /* stream tags */
 };
@@ -69,6 +69,8 @@ struct _GstFFMpegDemux
 
   GstFFStream *streams[MAX_STREAMS];
 
+  GstFlowCombiner *flowcombiner;
+
   gint videopads, audiopads;
 
   GstClockTime start_time;
@@ -282,6 +284,8 @@ gst_ffmpegdemux_init (GstFFMpegDemux * demux)
   demux->seek_event = NULL;
   gst_segment_init (&demux->segment, GST_FORMAT_TIME);
 
+  demux->flowcombiner = gst_flow_combiner_new ();
+
   /* push based data */
   g_mutex_init (&demux->ffpipe.tlock);
   g_cond_init (&demux->ffpipe.cond);
@@ -301,6 +305,8 @@ gst_ffmpegdemux_finalize (GObject * object)
 
   demux = (GstFFMpegDemux *) object;
 
+  gst_flow_combiner_free (demux->flowcombiner);
+
   g_mutex_clear (&demux->ffpipe.tlock);
   g_cond_clear (&demux->ffpipe.cond);
   gst_object_unref (demux->ffpipe.adapter);
@@ -326,8 +332,10 @@ gst_ffmpegdemux_close (GstFFMpegDemux * demux)
 
     stream = demux->streams[n];
     if (stream) {
-      if (stream->pad)
+      if (stream->pad) {
+        gst_flow_combiner_remove_pad (demux->flowcombiner, stream->pad);
         gst_element_remove_pad (GST_ELEMENT (demux), stream->pad);
+      }
       if (stream->tags)
         gst_tag_list_unref (stream->tags);
       g_free (stream);
@@ -602,15 +610,9 @@ gst_ffmpegdemux_perform_seek (GstFFMpegDemux * demux, GstEvent * event)
 
   /* and prepare to continue streaming */
   if (flush) {
-    gint n;
-
     /* send flush stop, peer will accept data and events again. We
      * are not yet providing data as we still have the STREAM_LOCK. */
     gst_ffmpegdemux_push_event (demux, gst_event_new_flush_stop (TRUE));
-    for (n = 0; n < MAX_STREAMS; ++n) {
-      if (demux->streams[n])
-        demux->streams[n]->last_flow = GST_FLOW_OK;
-    }
   }
   /* if successfull seek, we update our real segment and push
    * out the new segment. */
@@ -632,6 +634,7 @@ gst_ffmpegdemux_perform_seek (GstFFMpegDemux * demux, GstEvent * event)
 
   /* Mark discont on all srcpads and remove eos */
   gst_ffmpegdemux_set_flags (demux, TRUE, FALSE);
+  gst_flow_combiner_reset (demux->flowcombiner);
 
   /* and restart the task in case it got paused explicitely or by
    * the FLUSH_START event we pushed out. */
@@ -655,12 +658,8 @@ static gboolean
 gst_ffmpegdemux_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
 {
   GstFFMpegDemux *demux;
-  GstFFStream *stream;
   gboolean res = TRUE;
 
-  if (!(stream = gst_pad_get_element_private (pad)))
-    return FALSE;
-
   demux = (GstFFMpegDemux *) parent;
 
   switch (GST_EVENT_TYPE (event)) {
@@ -748,8 +747,8 @@ gst_ffmpegdemux_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
           break;
         case GST_FORMAT_DEFAULT:
           gst_query_set_position (query, GST_FORMAT_DEFAULT,
-              gst_util_uint64_scale (timeposition, avstream->r_frame_rate.num,
-                  GST_SECOND * avstream->r_frame_rate.den));
+              gst_util_uint64_scale (timeposition, avstream->avg_frame_rate.num,
+                  GST_SECOND * avstream->avg_frame_rate.den));
           res = TRUE;
           break;
         case GST_FORMAT_BYTES:
@@ -785,8 +784,8 @@ gst_ffmpegdemux_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
           break;
         case GST_FORMAT_DEFAULT:
           gst_query_set_duration (query, GST_FORMAT_DEFAULT,
-              gst_util_uint64_scale (timeduration, avstream->r_frame_rate.num,
-                  GST_SECOND * avstream->r_frame_rate.den));
+              gst_util_uint64_scale (timeduration, avstream->avg_frame_rate.num,
+                  GST_SECOND * avstream->avg_frame_rate.den));
           res = TRUE;
           break;
         case GST_FORMAT_BYTES:
@@ -865,8 +864,8 @@ gst_ffmpegdemux_src_convert (GstPad * pad,
       switch (*dest_fmt) {
         case GST_FORMAT_DEFAULT:
           *dest_value = gst_util_uint64_scale (src_value,
-              avstream->r_frame_rate.num,
-              GST_SECOND * avstream->r_frame_rate.den);
+              avstream->avg_frame_rate.num,
+              GST_SECOND * avstream->avg_frame_rate.den);
           break;
         default:
           res = FALSE;
@@ -877,8 +876,8 @@ gst_ffmpegdemux_src_convert (GstPad * pad,
       switch (*dest_fmt) {
         case GST_FORMAT_TIME:
           *dest_value = gst_util_uint64_scale (src_value,
-              GST_SECOND * avstream->r_frame_rate.num,
-              avstream->r_frame_rate.den);
+              GST_SECOND * avstream->avg_frame_rate.num,
+              avstream->avg_frame_rate.den);
           break;
         default:
           res = FALSE;
@@ -894,34 +893,6 @@ gst_ffmpegdemux_src_convert (GstPad * pad,
 }
 #endif
 
-static GstFlowReturn
-gst_ffmpegdemux_aggregated_flow (GstFFMpegDemux * demux)
-{
-  gint n;
-  GstFlowReturn res = GST_FLOW_OK;
-  gboolean have_ok = FALSE;
-
-  for (n = 0; n < MAX_STREAMS; n++) {
-    GstFFStream *s = demux->streams[n];
-
-    if (s) {
-      res = MIN (res, s->last_flow);
-
-      if (s->last_flow == GST_FLOW_OK)
-        have_ok = TRUE;
-    }
-  }
-
-  /* NOT_LINKED is OK, if at least one pad is linked */
-  if (res == GST_FLOW_NOT_LINKED && have_ok)
-    res = GST_FLOW_OK;
-
-  GST_DEBUG_OBJECT (demux, "Returning aggregated value of %s",
-      gst_flow_get_name (res));
-
-  return res;
-}
-
 static gchar *
 gst_ffmpegdemux_create_padname (const gchar * templ, gint n)
 {
@@ -968,7 +939,6 @@ gst_ffmpegdemux_get_stream (GstFFMpegDemux * demux, AVStream * avstream)
   stream->discont = TRUE;
   stream->avstream = avstream;
   stream->last_ts = GST_CLOCK_TIME_NONE;
-  stream->last_flow = GST_FLOW_OK;
   stream->tags = NULL;
 
   switch (ctx->codec_type) {
@@ -1054,6 +1024,7 @@ gst_ffmpegdemux_get_stream (GstFFMpegDemux * demux, AVStream * avstream)
 
   /* activate and add */
   gst_element_add_pad (GST_ELEMENT (demux), pad);
+  gst_flow_combiner_add_pad (demux->flowcombiner, pad);
 
   /* metadata */
   if ((codec = gst_ffmpeg_get_codecid_longname (ctx->codec_id))) {
@@ -1185,18 +1156,21 @@ gst_ffmpegdemux_open (GstFFMpegDemux * demux)
   else
     res = gst_ffmpeg_pipe_open (&demux->ffpipe, AVIO_FLAG_READ, &iocontext);
 
+  if (res < 0)
+    goto beach;
+
   demux->context = avformat_alloc_context ();
   demux->context->pb = iocontext;
   res = avformat_open_input (&demux->context, NULL, oclass->in_plugin, NULL);
 
   GST_DEBUG_OBJECT (demux, "av_open_input returned %d", res);
   if (res < 0)
-    goto open_failed;
+    goto beach;
 
   res = gst_ffmpeg_av_find_stream_info (demux->context);
   GST_DEBUG_OBJECT (demux, "av_find_stream_info returned %d", res);
   if (res < 0)
-    goto no_info;
+    goto beach;
 
   n_streams = demux->context->nb_streams;
   GST_DEBUG_OBJECT (demux, "we have %d streams", n_streams);
@@ -1278,13 +1252,7 @@ gst_ffmpegdemux_open (GstFFMpegDemux * demux)
   return TRUE;
 
   /* ERRORS */
-open_failed:
-  {
-    GST_ELEMENT_ERROR (demux, LIBRARY, FAILED, (NULL),
-        ("%s", gst_ffmpegdemux_averror (res)));
-    return FALSE;
-  }
-no_info:
+beach:
   {
     GST_ELEMENT_ERROR (demux, LIBRARY, FAILED, (NULL),
         ("%s", gst_ffmpegdemux_averror (res)));
@@ -1363,6 +1331,7 @@ gst_ffmpegdemux_loop (GstFFMpegDemux * demux)
   GstClockTime timestamp, duration;
   gint outsize;
   gboolean rawvideo;
+  GstFlowReturn stream_last_flow;
 
   /* open file if we didn't so already */
   if (!demux->opened)
@@ -1434,9 +1403,6 @@ gst_ffmpegdemux_loop (GstFFMpegDemux * demux)
 
   outbuf = gst_buffer_new_and_alloc (outsize);
 
-  if ((ret = gst_ffmpegdemux_aggregated_flow (demux)) != GST_FLOW_OK)
-    goto no_buffer;
-
   /* copy the data from packet into the target buffer
    * and do conversions for raw video packets */
   if (rawvideo) {
@@ -1449,7 +1415,7 @@ gst_ffmpegdemux_loop (GstFFMpegDemux * demux)
       src.data[0] = pkt.data;
       src.data[1] = NULL;
       src.data[2] = NULL;
-      src.linesize[0] = avstream->codec->width * 3;;
+      src.linesize[0] = avstream->codec->width * 3;
     } else {
       GST_WARNING ("Unknown demuxer %s, no idea what to do", plugin_name);
       gst_ffmpeg_avpicture_fill (&src, pkt.data,
@@ -1488,13 +1454,13 @@ gst_ffmpegdemux_loop (GstFFMpegDemux * demux)
       "Sending out buffer time:%" GST_TIME_FORMAT " size:%" G_GSIZE_FORMAT,
       GST_TIME_ARGS (timestamp), gst_buffer_get_size (outbuf));
 
-  ret = stream->last_flow = gst_pad_push (srcpad, outbuf);
+  ret = stream_last_flow = gst_pad_push (srcpad, outbuf);
 
   /* if a pad is in e.g. WRONG_STATE, we want to pause to unlock the STREAM_LOCK */
-  if ((ret != GST_FLOW_OK)
-      && ((ret = gst_ffmpegdemux_aggregated_flow (demux)) != GST_FLOW_OK)) {
+  if (((ret = gst_flow_combiner_update_flow (demux->flowcombiner,
+                  ret)) != GST_FLOW_OK)) {
     GST_WARNING_OBJECT (demux, "stream_movi flow: %s / %s",
-        gst_flow_get_name (stream->last_flow), gst_flow_get_name (ret));
+        gst_flow_get_name (stream_last_flow), gst_flow_get_name (ret));
     goto pause;
   }
 
@@ -1586,11 +1552,6 @@ drop:
       goto done;
     }
   }
-no_buffer:
-  {
-    av_free_packet (&pkt);
-    goto pause;
-  }
 }
 
 
@@ -1747,6 +1708,7 @@ gst_ffmpegdemux_sink_activate (GstPad * sinkpad, GstObject * parent)
 {
   GstQuery *query;
   gboolean pull_mode;
+  GstSchedulingFlags flags;
 
   query = gst_query_new_scheduling ();
 
@@ -1757,6 +1719,11 @@ gst_ffmpegdemux_sink_activate (GstPad * sinkpad, GstObject * parent)
 
   pull_mode = gst_query_has_scheduling_mode_with_flags (query,
       GST_PAD_MODE_PULL, GST_SCHEDULING_FLAG_SEEKABLE);
+
+  gst_query_parse_scheduling (query, &flags, NULL, NULL, NULL);
+  if (flags & GST_SCHEDULING_FLAG_SEQUENTIAL)
+    pull_mode = FALSE;
+
   gst_query_unref (query);
 
   if (!pull_mode)
