@@ -120,7 +120,7 @@ static void gst_ffmpegmux_set_property (GObject * object, guint prop_id,
 static void gst_ffmpegmux_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
-static GstCaps *gst_ffmpegmux_get_id_caps (enum CodecID *id_list);
+static GstCaps *gst_ffmpegmux_get_id_caps (enum AVCodecID *id_list);
 static void gst_ffmpeg_mux_simple_caps_set_int_list (GstCaps * caps,
     const gchar * field, guint num, const gint * values);
 
@@ -193,7 +193,7 @@ gst_ffmpegmux_base_init (gpointer g_class)
   GstPadTemplate *videosinktempl, *audiosinktempl, *srctempl;
   AVOutputFormat *in_plugin;
   GstCaps *srccaps, *audiosinkcaps, *videosinkcaps;
-  enum CodecID *video_ids = NULL, *audio_ids = NULL;
+  enum AVCodecID *video_ids = NULL, *audio_ids = NULL;
   gchar *longname, *description;
   const char *replacement;
   gboolean is_formatter;
@@ -646,6 +646,23 @@ gst_ffmpegmux_collected (GstCollectPads * pads, gpointer user_data)
       open_flags |= GST_FFMPEG_URL_STREAMHEADER;
     }
 
+    /* some house-keeping for downstream before starting data flow */
+    /* stream-start (FIXME: create id based on input ids) */
+    {
+      gchar s_id[32];
+
+      g_snprintf (s_id, sizeof (s_id), "avmux-%08x", g_random_int ());
+      gst_pad_push_event (ffmpegmux->srcpad, gst_event_new_stream_start (s_id));
+    }
+    /* segment */
+    {
+      GstSegment segment;
+
+      /* let downstream know we think in BYTES and expect to do seeking later on */
+      gst_segment_init (&segment, GST_FORMAT_BYTES);
+      gst_pad_push_event (ffmpegmux->srcpad, gst_event_new_segment (&segment));
+    }
+
     if (gst_ffmpegdata_open (ffmpegmux->srcpad, open_flags,
             &ffmpegmux->context->pb) < 0) {
       GST_ELEMENT_ERROR (ffmpegmux, LIBRARY, TOO_LAZY, (NULL),
@@ -737,9 +754,9 @@ gst_ffmpegmux_collected (GstCollectPads * pads, gpointer user_data)
 
       gst_buffer_map (buf, &map, GST_MAP_READ);
       gst_ffmpeg_avpicture_fill (&src, map.data,
-          PIX_FMT_RGB24, st->codec->width, st->codec->height);
+          AV_PIX_FMT_RGB24, st->codec->width, st->codec->height);
 
-      av_picture_copy (&dst, &src, PIX_FMT_RGB24,
+      av_picture_copy (&dst, &src, AV_PIX_FMT_RGB24,
           st->codec->width, st->codec->height);
       gst_buffer_unmap (buf, &map);
     } else {
@@ -823,7 +840,7 @@ gst_ffmpegmux_change_state (GstElement * element, GstStateChange transition)
 }
 
 static GstCaps *
-gst_ffmpegmux_get_id_caps (enum CodecID *id_list)
+gst_ffmpegmux_get_id_caps (enum AVCodecID *id_list)
 {
   GstCaps *caps, *t;
   gint i;
