@@ -246,6 +246,7 @@ gst_ffmpegviddec_class_init (GstFFMpegVidDecClass * klass)
   viddec_class->stop = gst_ffmpegviddec_stop;
   viddec_class->flush = gst_ffmpegviddec_flush;
   viddec_class->finish = gst_ffmpegviddec_finish;
+  viddec_class->drain = gst_ffmpegviddec_finish;        /* drain and finish are the same to us */
   viddec_class->decide_allocation = gst_ffmpegviddec_decide_allocation;
   viddec_class->propose_allocation = gst_ffmpegviddec_propose_allocation;
 }
@@ -1025,6 +1026,7 @@ gst_ffmpegviddec_negotiate (GstFFMpegVidDec * ffmpegdec,
       fps_d = 1;
     }
   }
+
   GST_LOG_OBJECT (ffmpegdec, "setting framerate: %d/%d", fps_n, fps_d);
   out_info->fps_n = fps_n;
   out_info->fps_d = fps_d;
@@ -1032,6 +1034,14 @@ gst_ffmpegviddec_negotiate (GstFFMpegVidDec * ffmpegdec,
   /* calculate and update par now */
   gst_ffmpegviddec_update_par (ffmpegdec, in_info, out_info);
 
+  /* Copy stereo/multiview info from upstream if set */
+  if (GST_VIDEO_INFO_MULTIVIEW_MODE (in_info) != GST_VIDEO_MULTIVIEW_MODE_NONE) {
+    GST_VIDEO_INFO_MULTIVIEW_MODE (out_info) =
+        GST_VIDEO_INFO_MULTIVIEW_MODE (in_info);
+    GST_VIDEO_INFO_MULTIVIEW_FLAGS (out_info) =
+        GST_VIDEO_INFO_MULTIVIEW_FLAGS (in_info);
+  }
+
   if (!gst_video_decoder_negotiate (GST_VIDEO_DECODER (ffmpegdec)))
     goto negotiate_failed;
 
@@ -1072,12 +1082,22 @@ gst_ffmpegviddec_do_qos (GstFFMpegVidDec * ffmpegdec,
     GstVideoCodecFrame * frame, gboolean * mode_switch)
 {
   GstClockTimeDiff diff;
+  GstSegmentFlags skip_flags =
+      GST_VIDEO_DECODER_INPUT_SEGMENT (ffmpegdec).flags;
 
   *mode_switch = FALSE;
 
   if (frame == NULL)
     return;
 
+  if (skip_flags & GST_SEGMENT_FLAG_TRICKMODE_KEY_UNITS) {
+    ffmpegdec->context->skip_frame = AVDISCARD_NONKEY;
+    *mode_switch = TRUE;
+  } else if (skip_flags & GST_SEGMENT_FLAG_TRICKMODE) {
+    ffmpegdec->context->skip_frame = AVDISCARD_NONREF;
+    *mode_switch = TRUE;
+  }
+
   diff =
       gst_video_decoder_get_max_decode_time (GST_VIDEO_DECODER (ffmpegdec),
       frame);
@@ -1088,17 +1108,19 @@ gst_ffmpegviddec_do_qos (GstFFMpegVidDec * ffmpegdec,
 
   GST_DEBUG_OBJECT (ffmpegdec, "decoding time %" G_GINT64_FORMAT, diff);
 
-  if (diff > 0 && ffmpegdec->context->skip_frame != AVDISCARD_DEFAULT) {
-    ffmpegdec->context->skip_frame = AVDISCARD_DEFAULT;
-    *mode_switch = TRUE;
-    GST_DEBUG_OBJECT (ffmpegdec, "QOS: normal mode");
-  }
+  if (*mode_switch == FALSE) {
+    if (diff > 0 && ffmpegdec->context->skip_frame != AVDISCARD_DEFAULT) {
+      ffmpegdec->context->skip_frame = AVDISCARD_DEFAULT;
+      *mode_switch = TRUE;
+      GST_DEBUG_OBJECT (ffmpegdec, "QOS: normal mode");
+    }
 
-  else if (diff <= 0 && ffmpegdec->context->skip_frame != AVDISCARD_NONREF) {
-    ffmpegdec->context->skip_frame = AVDISCARD_NONREF;
-    *mode_switch = TRUE;
-    GST_DEBUG_OBJECT (ffmpegdec,
-        "QOS: hurry up, diff %" G_GINT64_FORMAT " >= 0", diff);
+    else if (diff <= 0 && ffmpegdec->context->skip_frame != AVDISCARD_NONREF) {
+      ffmpegdec->context->skip_frame = AVDISCARD_NONREF;
+      *mode_switch = TRUE;
+      GST_DEBUG_OBJECT (ffmpegdec,
+          "QOS: hurry up, diff %" G_GINT64_FORMAT " >= 0", diff);
+    }
   }
 }
 
@@ -1114,6 +1136,9 @@ get_output_buffer (GstFFMpegVidDec * ffmpegdec, GstVideoCodecFrame * frame)
 
   GST_LOG_OBJECT (ffmpegdec, "get output buffer");
 
+  if (!ffmpegdec->output_state)
+    goto not_negotiated;
+
   ret =
       gst_video_decoder_allocate_output_frame (GST_VIDEO_DECODER (ffmpegdec),
       frame);
@@ -1154,9 +1179,14 @@ get_output_buffer (GstFFMpegVidDec * ffmpegdec, GstVideoCodecFrame * frame)
   /* special cases */
 alloc_failed:
   {
-    GST_DEBUG_OBJECT (ffmpegdec, "pad_alloc failed");
+    GST_DEBUG_OBJECT (ffmpegdec, "allocation failed");
     return ret;
   }
+not_negotiated:
+  {
+    GST_DEBUG_OBJECT (ffmpegdec, "not negotiated");
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
 }
 
 static void
@@ -1238,8 +1268,15 @@ gst_ffmpegviddec_video_frame (GstFFMpegVidDec * ffmpegdec,
     GST_DEBUG_OBJECT (ffmpegdec, "copy pal %p %p", &packet, pal);
   }
 
+  /* This might call into get_buffer() from another thread,
+   * which would cause a deadlock. Release the lock here
+   * and taking it again later seems safe
+   * See https://bugzilla.gnome.org/show_bug.cgi?id=726020
+   */
+  GST_VIDEO_DECODER_STREAM_UNLOCK (ffmpegdec);
   len = avcodec_decode_video2 (ffmpegdec->context,
       ffmpegdec->picture, have_data, &packet);
+  GST_VIDEO_DECODER_STREAM_LOCK (ffmpegdec);
 
   GST_DEBUG_OBJECT (ffmpegdec, "after decode: len %d, have_data %d",
       len, *have_data);
@@ -1439,16 +1476,14 @@ gst_ffmpegviddec_drain (GstFFMpegVidDec * ffmpegdec)
 
   if (oclass->in_plugin->capabilities & CODEC_CAP_DELAY) {
     gint have_data, len;
+    GstFlowReturn ret;
 
     GST_LOG_OBJECT (ffmpegdec,
         "codec has delay capabilities, calling until ffmpeg has drained everything");
 
     do {
-      GstFlowReturn ret;
-
       len = gst_ffmpegviddec_frame (ffmpegdec, NULL, 0, &have_data, NULL, &ret);
-
-    } while (len >= 0 && have_data == 1);
+    } while (len >= 0 && have_data == 1 && ret == GST_FLOW_OK);
     avcodec_flush_buffers (ffmpegdec->context);
   }
 }
@@ -1471,7 +1506,8 @@ gst_ffmpegviddec_handle_frame (GstVideoDecoder * decoder,
       GST_TIME_ARGS (frame->pts), GST_TIME_ARGS (frame->duration));
 
   if (!gst_buffer_map (frame->input_buffer, &minfo, GST_MAP_READ)) {
-    GST_ERROR_OBJECT (ffmpegdec, "Failed to map buffer");
+    GST_ELEMENT_ERROR (ffmpegdec, STREAM, DECODE, ("Decoding problem"),
+        ("Failed to map buffer for reading"));
     return GST_FLOW_ERROR;
   }
 
@@ -1521,18 +1557,18 @@ gst_ffmpegviddec_handle_frame (GstVideoDecoder * decoder,
     len =
         gst_ffmpegviddec_frame (ffmpegdec, data, size, &have_data, frame, &ret);
 
-    if (do_padding) {
-      memcpy (data + size, tmp_padding, FF_INPUT_BUFFER_PADDING_SIZE);
-    }
-
     if (ret != GST_FLOW_OK) {
       GST_LOG_OBJECT (ffmpegdec, "breaking because of flow ret %s",
           gst_flow_get_name (ret));
-      /* bad flow retun, make sure we discard all data and exit */
+      /* bad flow return, make sure we discard all data and exit */
       bsize = 0;
       break;
     }
 
+    if (do_padding) {
+      memcpy (data + size, tmp_padding, FF_INPUT_BUFFER_PADDING_SIZE);
+    }
+
     if (len == 0 && have_data == 0) {
       /* nothing was decoded, this could be because no data was available or
        * because we were skipping frames.
